# Aproximarea Functiilor Neliniare folosind Retele Neuronale
# Ciprian Pungila
# Univ. Politehnica, Fac. de Automatica si Calculatoare
# Anul III
# Version 1.3
# Compiled with Python 2.4, wxPython & NumPy

from wxPython.wx               import *
from wxPython.lib.wxPlotCanvas import *
from wxPython.lib              import wxPlotCanvas
from wxPython.lib.rcsizer      import RowColSizer

import random
import math
import Numeric

def sign(x):
    if x >= 0 :
        return 1
    else:
        return -1

def exp(x):
    if x > 25 :
        return math.exp(25)
    else:
        if x < -25:
            return math.exp(-25)
        else:
            return math.exp(x)

def _InitObjects():
    # this initial representation is only for intro purposes,
    # it does not affect the program in other ways
    
    # 100 points sin function, plotted as green circles
    data1 = 2.*Numeric.pi*Numeric.arange(200)/200.
    data1.shape = (100, 2)
    data1[:,1] = Numeric.sin(data1[:,0])
    markers1 = PolyMarker(data1, color='green', marker='circle',size=1)

    # 50 points cos function, plotted as red line
    data1 = 2.*Numeric.pi*Numeric.arange(100)/100.
    data1.shape = (50,2)
    data1[:,1] = Numeric.cos(data1[:,0])
    lines = PolyLine(data1, color='red')

    # A few more points...
    pi = Numeric.pi
    markers2 = PolyMarker([(0., 0.), (pi/4., 1.), (pi/2, 0.),
                          (3.*pi/4., -1)], color='blue',
                          fillcolor='green', marker='cross')

    return PlotGraphics([markers1, lines, markers2])

    # the class uses to build our neural network
class network:
    # the functions used
    functions = {"logistica": lambda x: (1/(1+exp(-x))), 
                    "tanh": lambda x: ((exp(2*x)-1)/(exp(2*x)+1)), 
                    "liniara": lambda x: x,
            }

    hfunctions = {"logistica": lambda x: (1/(1+exp(-x))), 
                    "tanh": lambda x: ((exp(2*x)-1)/(exp(2*x)+1)), 
            }

    # their derivate
    derivate_functions = {"logistica": lambda x: (exp(-x)/(2*exp(-x)+exp(-2*x)+1)), 
                    "tanh": lambda x: (4*exp(2*x)/(2*exp(2*x)+exp(4*x)+1)), 
                    "liniara": lambda x: 1,
            }

    hderivate_functions = {"logistica": lambda x: (exp(-x)/(2*exp(-x)+exp(-2*x)+1)), 
                    "tanh": lambda x: (4*exp(2*x)/(2*exp(2*x)+exp(4*x)+1)), 
            }

    # this is the default constructor
    def __init__(self, mainwin, l, maxerror, epochs, k, function = "sinus", output_function = "logistica", hidden_function = "logistica", a=-5, b=5, eta = 0.1, magnitude = 0.05):
        self.mainwin = mainwin
        self.w1_0 = [(random.random()-0.5)*2 for i in range(1, k+1, 1) ] 
        self.w1_1 = [(random.random()-0.5)*2 for i in range(1, k+1, 1) ] 
        self.w2 = [random.random() for i in range(1, k+1, 1) ]
        self.x1 = [0 for i in range(1, k+1, 1)]
        self.y1 = [0 for i in range(1, k+1, 1)]
        self.x2 = [0 for i in range(1, k+1, 1)]
        self.y2 = [0 for i in range(1, k+1, 1)]
        self.l = l # number of steps
        self.h = abs(b-a) / l # this is the actual step
        self.eta = eta # the learning rate
        self.a = a # = a
        self.b = b # = b
        self.output_function = output_function # for the output level
        self.hidden_function = hidden_function # for the hidden level
        self.function = self.mainwin.functions[function] # the function to estimate
        self.magnitude = magnitude # magnitude of noise
        self.k = k # number of hidden units
        self.p = 0 # number of training epochs
        self.w2_0 = random.random() # this is w2[0] in C
        self.maxerror = maxerror # maximum tolerated error
        self.epochs = epochs # number of maximum epochs
        self.error = 1000 # currently calculated error

        self.x = [self.a + (i-1)*self.h for i in range(1, self.l+1, 1)]
        self.d = [self.function(self.x[i-1]) + self.magnitude*(2*random.random() - 1) for i in range(1, self.l+1, 1)]
        

    # each time we call an instance of the class (e.g. network()), this method is called
    def __call__(self):
        return

    # the output function
    def output_f(self, x):
        return self.functions[self.output_function](x)

    # the k function
    def houtput_f(self, x):
        return self.hfunctions[self.hidden_function](x)

    # derivate of the output function
    def output_df(self, x):
        return self.derivate_functions[self.output_function](x)

    # derivate of the hidden function
    def houtput_df(self, x):
        return self.hderivate_functions[self.hidden_function](x)

    # compute estimated function value in xx using neural network
    def compute_estimated(self, x):
        for j in range(1, self.k+1, 1):
            self.x1[j-1] = self.w1_1[j-1]*x - self.w1_0[j-1]
            self.y1[j-1] = self.houtput_f(self.x1[j-1])
        self.x2 = 0
        for j in range(1, self.k+1, 1):
            self.x2 = self.x2 + self.w2[j-1]*self.y1[j-1] - self.w2_0
        self.y2 = self.output_f(self.x2)
        return self.y2

    # compute estimated function value in xx using neural network
    def compute_estimated_aux(self, x):
        x1 = self.x1
        y1 = self.y1
        w1_1 = self.w1_1
        w1_0 = self.w1_0
        h_output_f = self.houtput_f
        x1 = self.x1
        x2 = self.x2
        for j in range(1, self.k+1, 1):
            x1[j-1] = w1_1[j-1]*x - w1_0[j-1]
            y1[j-1] = h_output_f(x1[j-1])
        x2 = 0
        w2 = self.w2
        w2_0 = self.w2_0
        y2 = self.y2
        my_output_f = self.output_f
        for j in range(1, self.k+1, 1):
            x2 = x2 + w2[j-1]*y1[j-1] - w2_0
        y2 = my_output_f(x2)
        return y2

    # learning function
    def learn(self):
        self.delta1 = [0 for i in range(1, 20+1, 1)]
        self.delta2 = 0
        self.error = 1000
        dlg = wxProgressDialog("Antrenare retea",
                       "Reteaua este antrenata",
                       self.epochs,
                       self.mainwin, # instance of type (wxWindow *)
                       wxPD_CAN_ABORT | wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME | wxPD_ESTIMATED_TIME)
        keepGoing = true
        while keepGoing and (self.p <= self.epochs) and (abs(self.error) > self.maxerror):
            for i in range(1, self.l+1, 1):
                # forward
                for j in range(1, self.k+1, 1):
                    self.x1[j-1] = self.w1_1[j-1]*self.x[i-1] - self.w1_0[j-1]
                    self.y1[j-1] = self.houtput_f(self.x1[j-1])
                self.x2 = 0
                for j in range(1, self.k+1, 1):
                    self.x2 = self.x2 + self.w2[j-1]*self.y1[j-1] - self.w2_0
                self.y2 = self.output_f(self.x2)
                # backward
                self.delta2 = self.output_df(self.x2)*(self.d[i-1] - self.y2)
                for j in range(1, self.k+1, 1):
                    self.delta1[j-1] = self.houtput_df(self.x1[j-1])*self.w2[j-1]*self.delta2
                # adjusting
                for j in range(1, self.k+1, 1):
                    self.w2[j-1] = self.w2[j-1] + self.eta*self.y1[j-1]*self.delta2
                    self.w1_1[j-1] = self.w1_1[j-1] + self.eta*self.x[i-1]*self.delta1[j-1]
                    self.w1_0[j-1] = self.w1_0[j-1] - self.eta*self.delta1[j-1]
                self.w2_0 = self.w2_0 - self.eta*self.delta2
            # error calculation
            self.error = 0
            for i in range(1, self.l+1, 1):
                self.aux = 0
                for j in range(1, self.k+1, 1):
                    self.aux = self.aux + self.w2[j-1]*self.houtput_f(self.w1_1[j-1]*self.x[i-1] - self.w1_0[j-1]) - self.w2_0
                self.error += (self.d[i-1] - self.output_f(self.aux))*(self.d[i-1] - self.output_f(self.aux))
            self.p += 1

            self.mainwin.error_pg.add_point(self.p, self.error) # add point for redrawing the error graph
            self.mainwin.pg.make_funcs_aux() # add point for redrawing the estimated function graph
 
            self.mainwin.plot.redraw()
            self.mainwin.error_plot.redraw()
            self.mainwin.update_probes()
            keepGoing = dlg.Update(min(self.p, self.epochs))
        # now everything has been calculated, time to show the
        # output graphs
        dlg.Destroy()
        

class nn_errorplotcanvas(PlotCanvas):
    def __init__(self, parent):
        PlotCanvas.__init__(self, parent)

class nn_errorplotgraphics(PlotGraphics):
    point_q = 50.0
    def __init__(self):
        self.objects = []
        self.points = []
        self.last_scale = [1,1]
        self.last_shift = [0,0]

        self.objects.append( PolyLine(self.points, color="red") )

    def add_point(self, x, y):
        self.points.append( [x,y] )
        self.objects = []
        self.objects.append( PolyLine(self.points, color="red") )

    def scaleAndShift(self, scale=1, shift=0):
        self.last_scale = scale
        self.last_shift = shift
        PlotGraphics.scaleAndShift(self, scale, shift)
        
class nn_plotcanvas(PlotCanvas):
    def __init__(self, parent, error_plot):
        PlotCanvas.__init__(self, parent)
        self.error_plot = error_plot
        self.parent = parent
        
        
class nn_plotgraphics(PlotGraphics):
    point_q = 50.0
    # default constructor
    def __init__(self, network, target, error_pg = None):
        self.objects = [] 
        self.run = 0
        self.target = target
        self.network = network
        self.error_pg = error_pg
        self.make_funcs()

    # make the functions to plot - auxiliary function  
    def make_funcs_aux(self):
        delme = []
        for o in self.objects:
            if isinstance(o, PolyLine):
                delme.append( o )
        for o in delme:
            self.objects.remove( o )
            
        net_points = []
        target_points = []
        step_size = abs(self.network.a-self.network.b)/self.point_q
        start = self.network.a
        for steps in range(self.point_q):
            x = start + steps*step_size
            y = self.network.compute_estimated_aux(x)
            net_points.append( (x, y) )
            y = self.target(x)
            target_points.append( (x,y) )
        self.objects.append(PolyLine(net_points, color="blue"))
        self.objects.append(PolyLine(target_points, color="green"))

    # make the functions to plot    
    def make_funcs(self):
        delme = []
        for o in self.objects:
            if isinstance(o, PolyLine):
                delme.append( o )
        for o in delme:
            self.objects.remove( o )
            
        net_points = []
        target_points = []
        step_size = abs(self.network.a-self.network.b)/self.point_q
        start = self.network.a
        for steps in range(self.point_q):
            x = start + steps*step_size
            y = self.network.compute_estimated(x)
            net_points.append( (x, y) )
            y = self.target(x)
            target_points.append( (x,y) )
        self.objects.append(PolyLine(net_points, color="blue"))
        self.objects.append(PolyLine(target_points, color="green"))

    # scale and shift the graphic
    def scaleAndShift(self, scale=1, shift=0):
        self.last_scale = scale
        self.last_shift = shift
        PlotGraphics.scaleAndShift(self, scale, shift)

        
class main_window(wxFrame):
    def __init__(self, parent, id):
        wxFrame.__init__(self, parent, -1, "Aproximarea functiilor neliniare cu retele neuronale", size = (800, 500),
                         style=wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE)


        # ------------------------------------------------------------------------------------
        # Set up menu bar for the program.
        # ------------------------------------------------------------------------------------
        self.mainmenu = wxMenuBar()   # Create menu bar.
        mainwindow = self

        menu=wxMenu()                                 # Make a menu (will be the Project menu)

        exitID=wxNewId()
        menu.Append(exitID, '&Nou...', 'Creeaza o retea noua')
        EVT_MENU(self, exitID, self.OnNew)
        
        exitID=wxNewId()
        menu.Append(exitID, '&Iesire', 'Iesire din program')
        EVT_MENU(self, exitID, self.OnExit)

        self.mainmenu.Append (menu, '&Fisier')       # Add the project menu to the menu bar.

        menu=wxMenu()                                 # Make a menu (will be the Project menu)

        exitID=wxNewId()
        menu.Append(exitID, '&Despre', 'Informatii')
        EVT_MENU(self, exitID, self.OnAbout)
        
        self.mainmenu.Append (menu, '&Ajutor')       # Add the project menu to the menu bar.



        self.SetMenuBar (self.mainmenu)               # Attach the menu bar to the window.
        self.CreateStatusBar()


  

        # ------------------------------------------------------------------------------------
        # Create horizontal the splitter window.
        # ------------------------------------------------------------------------------------
        hsid = wxNewId()
        self.hsplitter = hsplitter = wxSplitterWindow (self, hsid, style=wxNO_3D|wxSP_3D)
        #hsplitter.SetMinimumPaneSize (1)
        # ------------------------------------------------------------------------------------
        # Create the configuration panel
        # ------------------------------------------------------------------------------------

        self.cpanel = cpanel = wxPanel(hsplitter, -1)

        # ------------------------------------------------------------------------------------
        # Create the vertical splitter window.
        # ------------------------------------------------------------------------------------
        splitter = wxSplitterWindow (hsplitter, -1, style=wxNO_3D|wxSP_3D)
        splitter.update_probes = self.update_probes
        splitter.SetMinimumPaneSize (1)
        

        hsplitter.SplitHorizontally (self.cpanel, splitter)
        # ------------------------------------------------------------------------------------
        # Create the editor on the right.
        # ------------------------------------------------------------------------------------
        self.error_plot = win = nn_errorplotcanvas(splitter)
        win.draw(_InitObjects(),'automatic','automatic')
        
        # ------------------------------------------------------------------------------------
        # Create the plot canvas
        # ------------------------------------------------------------------------------------
        tID = wxNewId()
        self.plot =  win = nn_plotcanvas(splitter, self.error_plot)
        win.draw(_InitObjects(),'automatic','automatic')
        

        # ------------------------------------------------------------------------------------
        # Install the tree and the editor.
        # ------------------------------------------------------------------------------------
        splitter.SplitVertically (self.plot, self.error_plot)
        splitter.SetSashPosition (420, true)

        # place controls
        gridsz = RowColSizer()
        

        # new network
        line = 1
        c = wxStaticText(cpanel, -1, " Neuroni in nivelul ascuns: ")
        gridsz.Add(c, pos=(line,1))
        
        cbID = wxNewId()
        self.layers = 10
        c = wxComboBox(cpanel, cbID, str(self.layers), choices=["2", "3", "4", "5", "6", "7", "8", "9",
                                                                "10", "11", "12", "13", "14", "15", "16", "17",
                                                                "18", "19", "20"],size=(150,-1), style=wxCB_DROPDOWN)
        gridsz.Add(c, pos=(line,2))
        EVT_COMBOBOX(self, cbID, self.OnComboHidden)
        

        line += 1
        c = wxStaticText(cpanel, -1, " Functie iesire: ")
        gridsz.Add( c, pos=(line,1))

     
        cbID = wxNewId()
        self.output_function = "logistica"
        c = wxComboBox(cpanel, cbID, self.output_function, choices=network.functions.keys(),
                                     size=(150,-1), style=wxCB_DROPDOWN)
        gridsz.Add( c, pos=(line,2))                                     
        EVT_COMBOBOX(self, cbID, self.OnComboOutputFunction)                    

        line += 1
        c = wxStaticText(cpanel, -1, " Start: ")
        gridsz.Add( c, pos=(line,1))
        self.lrange_vals = ["0","-0.5","-1","-1.5","-2","-3","-5","-10","-50","-100"]
        self.lrange = "-5"
        cbID = wxNewId()
        c = wxComboBox(cpanel, cbID, self.lrange, choices=self.lrange_vals,
                                     size=(150,-1), style=wxCB_DROPDOWN)
        gridsz.Add( c, pos=(line,2))                                     
        EVT_COMBOBOX(self, cbID, self.OnComboLrange)       

        c = wxStaticText(cpanel, -1, " Stop: ")
        gridsz.Add( c, pos=(line,4))

        self.rrange_vals = ["0","0.5","1","1.5","2","3","5","10","50","100"]
        self.rrange = "5"
        cbID = wxNewId()
        c = wxComboBox(cpanel, cbID, self.rrange, choices=self.rrange_vals,
                                     size=(150,-1), style=wxCB_DROPDOWN)
        gridsz.Add( c, pos=(line,5))                                     
        EVT_COMBOBOX(self, cbID, self.OnComboRrange)       

        # input function
        line += 1
        c = wxStaticText(cpanel, -1, " Functie nivel ascuns: ")
        gridsz.Add( c, pos=(line,1))

        cbID = wxNewId()
        self.hidden_function = "logistica"
        c = wxComboBox(cpanel, cbID, self.hidden_function, choices=network.hfunctions.keys(),
                                     size=(150,-1), style=wxCB_DROPDOWN)
        gridsz.Add( c, pos=(line,2))                                     
        EVT_COMBOBOX(self, cbID, self.OnComboInputFunction)                    

        # other controls       
        line += 1
        c = wxStaticText(cpanel, -1, " Rata de invatare: ")
        gridsz.Add( c, pos=(line,1))
        self.alpha_vals = ["0.01","0.05","0.1","0.15","0.2","0.3"]
        self.eta = "0.1"
        
        cbID = wxNewId()
        c = wxComboBox(cpanel, cbID, self.eta, choices=self.alpha_vals,
                                     size=(150,-1), style=wxCB_DROPDOWN)
        gridsz.Add( c, pos=(line,2))                                     
        EVT_COMBOBOX(self, cbID, self.OnComboAlpha)       
        
        line += 1
        c = wxStaticText(cpanel, -1, " Functia: ")
        gridsz.Add( c, pos=(line,1))
        self.functions = { "1 (constanta)": lambda x: 1,
                            "x": lambda x: x,
                            "sign": sign,
                            "sinus": lambda x: math.sin(x),
                            "cosinus": lambda x: math.cos(x),
                            "x^2": lambda x: x*x,
                            "x^3": lambda x: x*x*x,
                            "x*sin x": lambda x: x*math.sin(x),
                            "x*sin(5*x)": lambda x: x*math.sin(5*x),
                            "x*x+1": lambda x: x*x+1,
                            "10*x*x+1": lambda x: 10*x*x+1
                            }
        self.function = "sinus"
        
        cbID = wxNewId()
        c = wxComboBox(cpanel, cbID, self.function, choices=self.functions.keys(),
                                     size=(150,-1), style=wxCB_DROPDOWN)
        gridsz.Add( c, pos=(line,2))                                     
        EVT_COMBOBOX(self, cbID, self.OnComboFunction)                                     

        line += 1
        cbID = wxNewId()
        c = wxButton(cpanel , cbID, "Nou")
        gridsz.Add( c, pos=(line,2))                                     
        EVT_BUTTON(self, cbID, self.OnNew)


        # train controls
        c = wxStaticText(cpanel, -1, " Pasi: ")
        gridsz.Add( c, pos=(1,4))                   
        
        cbID = wxNewId()
        self.steps = c = wxComboBox(cpanel, cbID, "5000", choices=["1", "2", "5", "10", "25", "50", "100", "250", "500","1000","5000", "25000", "100000"],
                                     size=(150,-1), style=wxCB_DROPDOWN)
        gridsz.Add( c, pos=(1,5))                                                        
        EVT_COMBOBOX(self, cbID, self.OnComboTrain)  

        cbID = wxNewId()
        c = wxButton(cpanel, cbID, "Antrenare")
        gridsz.Add( c, pos=(2,5))                                                                
        EVT_BUTTON(self, cbID, self.OnTrain)

        # add function response to manual sample
        c = wxStaticText(cpanel, -1, " Dati o valoare: ")
        gridsz.Add( c, pos=(6,4))                   

        self.point_to_compute = c = wxTextCtrl(cpanel, -1, "0")
        gridsz.Add( c, pos=(6,5))

        cbID = wxNewId()
        c = wxButton(cpanel, cbID, "Calculeaza")
        gridsz.Add( c, pos=(7,5))                                                                
        EVT_BUTTON(self, cbID, self.OnCalculate)

        c = wxStaticText(cpanel, -1, " Valoarea reala: ")
        gridsz.Add( c, pos=(6,6))                

        self.real_value = c = wxTextCtrl(cpanel, -1, "0", style = wxTE_READONLY)
        gridsz.Add( c, pos=(6,7))

        c = wxStaticText(cpanel, -1, " Valoarea calculata: ")
        gridsz.Add( c, pos=(7,6))                

        self.calculated_value = c = wxTextCtrl(cpanel, -1, "0", style = wxTE_READONLY)
        gridsz.Add( c, pos=(7,7))

        # probes
        c = wxStaticText(cpanel, -1, " Pasul curent: ")
        gridsz.Add( c, pos=(4,4))                   

        self.run_display = c = wxTextCtrl(cpanel, -1, "0", style = wxTE_READONLY)
        gridsz.Add( c, pos=(4,5))                  
        
        c = wxStaticText(cpanel, -1, " Magnitudinea: ")
        gridsz.Add( c, pos=(4,6))                   

        self.user_magnitude = c = wxTextCtrl(cpanel, -1, "0.05")
        gridsz.Add( c, pos=(4,7))                  

        c = wxStaticText(cpanel, -1, " Eroarea curenta: ")
        gridsz.Add( c, pos=(5,4))                   

        self.error_display = c = wxTextCtrl(cpanel, -1, "0", style = wxTE_READONLY)
        gridsz.Add( c, pos=(5,5))                  
        
        c = wxStaticText(cpanel, -1, " Toleranta invatare: ")
        gridsz.Add( c, pos=(3,6))                   

        self.max_error = c = wxTextCtrl(cpanel, -1, "0.01")
        gridsz.Add( c, pos=(3,7))                  

        c = wxStaticText(cpanel, -1, " Nr. exemple din set: ")
        gridsz.Add( c, pos=(2,6))                   

        self.nr_examples = c = wxTextCtrl(cpanel, -1, "10")
        gridsz.Add( c, pos=(2,7))                  


        self.cpanel.SetSizer(gridsz)
        self.cpanel.SetAutoLayout(true)
        self.Show(true)
        hsplitter.SetSashPosition(185)

    def OnComboHidden(self, evt):
        self.layers = int(evt.GetString())

    def OnComboFunction(self, evt):
        self.function = evt.GetString()

    def OnComboAlpha(self, evt):
        self.eta = evt.GetString()

    def OnComboRrange(self, evt):
        self.rrange = evt.GetString()

    def OnComboLrange(self, evt):
        self.lrange = evt.GetString()

    def OnComboOutputFunction(self, evt):
        self.output_function = evt.GetString()
        
    def OnComboInputFunction(self, evt):
        self.hidden_function = evt.GetString()

    def OnComboTrain(self, evt):
        return
        
    def OnExit(self, evt):
        self.Close(true)

    def OnAbout(self, event):
        dlg = wxMessageDialog(self, "Aproximarea functiilor neliniare cu retele neuronale\nCiprian Pungila, Fac. de Automatica si Calculatoare\nE-mail: cipy@email.com",
                              "Despre", wxOK | wxICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()


    # event generated when the neural network is being trained    
    def OnTrain(self, evt):
            self.network.learn()
            self.plot.redraw()
            self.error_plot.redraw()
            self.update_probes()

        
    # event generated when we need to compute an individual value    
    def OnCalculate(self, evt):
        try:
            to_compute = float(self.point_to_compute.GetValue())
            self.real_value.SetValue( str(self.pg.target(to_compute)) )
            self.calculated_value.SetValue( str(self.pg.network.compute_estimated(to_compute)) )
        except:
            msgdlg = wxMessageDialog(self, 'A aparut o eroare la calcul! Probabil un overflow matematic?',
                          'Eroare la calcul', wxOK | wxICON_EXCLAMATION)

            msgdlg.ShowModal()
            msgdlg.Destroy()




    # event generated when creating a new neural network
    def OnNew(self, evt):
        self.network = network(self, # mainwin
                               int(self.nr_examples.GetValue()), # l
                               float(self.max_error.GetValue()), # maxerror
                               int(self.steps.GetValue()), # epochs
                               self.layers, # k
                               function = self.function, # function to simulate
                               hidden_function = self.hidden_function, # function for output layer
                               output_function = self.output_function, # function for hidden layer
                               a = float(self.lrange), # a
                               b = float(self.rrange), # b
                               eta = float(self.eta), # eta (learning rate)
                               magnitude = float(self.user_magnitude.GetValue())) # magnitude of noise

        # create the necessary classes
        self.error_pg = nn_errorplotgraphics()
        self.pg = nn_plotgraphics(self.network, self.functions[self.function], error_pg = self.error_pg)        

        # draw the graphs
        self.plot.draw( self.pg, 'automatic', 'automatic' )
        y = 1.0
        self.error_pg.add_point(0, y)
        self.error_plot.draw( self.error_pg, 'automatic', 'automatic' )        


    def update_probes(self):
        self.error_display.SetValue( str(self.network.error) ) # updating the error editfield
        self.run_display.SetValue( str(self.network.p) )     # and the number of steps

class App(wxApp):
    def OnInit(self):
        frame = main_window(None, -1)
        self.SetTopWindow(frame)
        return true

app = App(0)
app.MainLoop()

